/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 19/12/22.
//

#ifndef FGGL_GRID_TOKENS_HPP
#define FGGL_GRID_TOKENS_HPP

#include <cstdint>
#include <map>

#include "fggl/util/guid.hpp"

namespace fggl::grid {

	using TokenIdentifer = fggl::util::OpaqueName<uint64_t, struct Token>;

	class TokenType {
		public:
			using PropType = int64_t;

			inline void setProperty(util::GUID name, PropType newVal) {
				m_properties[name] = newVal;
			}

			[[nodiscard]]
			inline PropType getProperty(util::GUID name, PropType unsetVal=0) const {
				try {
					return m_properties.at(name);
				} catch ( std::out_of_range& e ) {
					return unsetVal;
				}
			}
		private:
			std::map<util::GUID, PropType> m_properties;
			std::map<util::GUID, uint64_t> m_cost;
	};

	class Token {
		public:
			Token() = default;
			explicit Token(std::shared_ptr<TokenType> type) : m_type(std::move(type)) {}

			inline void setProperty(util::GUID name, TokenType::PropType newVal) {
				m_properties[name] = newVal;
			}

			[[nodiscard]]
			inline TokenType::PropType getProperty(util::GUID name, TokenType::PropType unsetVal=0) const {
				try {
					auto prop = m_properties.at(name);
					return prop;
				} catch ( std::out_of_range& ex) {
					return m_type->getProperty(name, unsetVal);
				}
			}

			[[nodiscard]]
			inline TokenType::PropType getPropertyDirect(util::GUID name, TokenType::PropType unsetVal=0) const {
				try {
					auto prop = m_properties.at(name);
					return prop;
				} catch ( std::out_of_range& ex) {
					return unsetVal;
				}
			}
		private:
			TokenIdentifer m_id;
			std::shared_ptr<TokenType> m_type;
			std::map<util::GUID, TokenType::PropType> m_properties;
	};

} // namespace fggl::grid

#endif //FGGL_GRID_TOKENS_HPP
